@@ -47,90 +47,31 @@
 #define N_(x) x
 #endif
 
-/*
- * Error handling
- */
-
 /* Returns TRUE if there was an error */
-static gboolean g_conf_handle_corba_exception(CORBA_Environment* ev);
+static gboolean g_conf_handle_corba_exception(CORBA_Environment* ev, GConfError** err);
 
-static gchar* last_details = NULL;
-static GConfErrNo last_errno = G_CONF_SUCCESS;
-
-static const gchar* err_msgs[11] = {
-  N_("Success"),
-  N_("Failed"),
-  N_("Configuration server couldn't be contacted"),
-  N_("Permission denied"),
-  N_("Couldn't resolve address for configuration source"),
-  N_("Bad key or directory name"),
-  N_("Parse error"),
-  N_("Type mismatch"),
-  N_("Key operation on directory"),
-  N_("Directory operation on key"),
-  N_("Can't overwrite existing read-only value")
-};
-
-static const int n_err_msgs = sizeof(err_msgs)/sizeof(err_msgs[0]);
-
-void         
-g_conf_clear_error(void)
+gboolean
+g_conf_key_check(const gchar* key, GConfError** err)
 {
-  if (last_details)
+  gchar* why = NULL;
+  if (!g_conf_valid_key(key, &why))
     {
-      g_free(last_details);
-      last_details = NULL;
+      if (err)
+        *err = g_conf_error_new(G_CONF_BAD_KEY, _("`%s': %s"),
+                                key, why);
+      g_free(why);
+      return FALSE;
     }
-  last_errno = G_CONF_SUCCESS;
-}
-
-void
-g_conf_set_error(GConfErrNo en, const gchar* fmt, ...)
-{
-  gchar* details;
-  va_list args;
-
-  if (last_details != NULL)
-    g_free(last_details);
-    
-  va_start (args, fmt);
-  details = g_strdup_vprintf(fmt, args);
-  va_end (args);
-
-  last_details = g_strconcat(g_conf_strerror(en), ":\n ", details, NULL);
-
-  last_errno = en;
-
-  g_free(details);
-}
-
-const gchar* 
-g_conf_error          (void)
-{
-  return last_details ? last_details : _("No error");
-}
-
-const gchar* 
-g_conf_strerror       (GConfErrNo en)
-{
-  g_return_val_if_fail (en < n_err_msgs, NULL);
-
-  return _(err_msgs[en]);    
-}
-
-GConfErrNo   
-g_conf_errno          (void)
-{
-  return last_errno;
+  return TRUE;
 }
 
 /* 
  * GConfPrivate
  */
 
-typedef struct _GConfPrivate GConfPrivate;
+typedef struct _GConfEnginePrivate GConfEnginePrivate;
 
-struct _GConfPrivate {
+struct _GConfEnginePrivate {
   ConfigServer_Context context;
   guint refcount;
 };
@@ -140,16 +81,16 @@ typedef struct _GConfCnxn GConfCnxn;
 struct _GConfCnxn {
   guint client_id;
   CORBA_unsigned_long server_id; /* id returned from server */
-  GConf* conf;     /* conf we're associated with */
+  GConfEngine* conf;     /* conf we're associated with */
   GConfNotifyFunc func;
   gpointer user_data;
 };
 
-static GConfCnxn* g_conf_cnxn_new(GConf* conf, CORBA_unsigned_long server_id, GConfNotifyFunc func, gpointer user_data);
+static GConfCnxn* g_conf_cnxn_new(GConfEngine* conf, CORBA_unsigned_long server_id, GConfNotifyFunc func, gpointer user_data);
 static void       g_conf_cnxn_destroy(GConfCnxn* cnxn);
 static void       g_conf_cnxn_notify(GConfCnxn* cnxn, const gchar* key, GConfValue* value);
 
-static ConfigServer g_conf_get_config_server(gboolean start_if_not_found);
+static ConfigServer g_conf_get_config_server(gboolean start_if_not_found, GConfError** err);
 static ConfigListener g_conf_get_config_listener(void);
 
 /* We'll use client-specific connection numbers to return to library
@@ -171,7 +112,7 @@ static CnxnTable* ctable_new(void);
 static void       ctable_insert(CnxnTable* ct, GConfCnxn* cnxn);
 static void       ctable_remove(CnxnTable* ct, GConfCnxn* cnxn);
 static void       ctable_remove_by_client_id(CnxnTable* ct, guint client_id);
-static GSList*    ctable_remove_by_conf(CnxnTable* ct, GConf* conf);
+static GSList*    ctable_remove_by_conf(CnxnTable* ct, GConfEngine* conf);
 static GConfCnxn* ctable_lookup_by_client_id(CnxnTable* ct, guint client_id);
 static GConfCnxn* ctable_lookup_by_server_id(CnxnTable* ct, CORBA_unsigned_long server_id);
 
@@ -180,29 +121,29 @@ static GConfCnxn* ctable_lookup_by_server_id(CnxnTable* ct, CORBA_unsigned_long
  *  Public Interface
  */
 
-GConf*
-g_conf_new            (void)
+GConfEngine*
+g_conf_engine_new            (void)
 {
-  GConfPrivate* priv;
+  GConfEnginePrivate* priv;
 
-  priv = g_new0(GConfPrivate, 1);
+  priv = g_new0(GConfEnginePrivate, 1);
 
   priv->context = ConfigServer_default_context;
   priv->refcount = 1;
   
-  return (GConf*) priv;
+  return (GConfEngine*) priv;
 }
 
-GConf*
-g_conf_new_from_address(const gchar* address)
+GConfEngine*
+g_conf_engine_new_from_address(const gchar* address, GConfError** err)
 {
-  GConf* gconf;
-  GConfPrivate* priv;
+  GConfEngine* gconf;
+  GConfEnginePrivate* priv;
   CORBA_Environment ev;
   ConfigServer cs;
   ConfigServer_Context ctx;
   
-  cs = g_conf_get_config_server(TRUE);
+  cs = g_conf_get_config_server(TRUE, err);
 
   if (cs == CORBA_OBJECT_NIL)
     return NULL; /* Error should already be set */
@@ -211,7 +152,7 @@ g_conf_new_from_address(const gchar* address)
   
   ctx = ConfigServer_get_context(cs, (gchar*)address, &ev);
 
-  if (g_conf_handle_corba_exception(&ev))
+  if (g_conf_handle_corba_exception(&ev, err))
     {
       /* FIXME we could do better here... maybe respawn the server if needed... */
       return NULL;
@@ -219,16 +160,17 @@ g_conf_new_from_address(const gchar* address)
   
   if (ctx == ConfigServer_invalid_context)
     {
-      g_conf_set_error(G_CONF_BAD_ADDRESS,
-                       _("Server couldn't resolve the address `%s'"),
-                       address);
+      if (err)
+        *err = g_conf_error_new(G_CONF_BAD_ADDRESS,
+                                _("Server couldn't resolve the address `%s'"),
+                                address);
 
       return NULL;
     }
   
-  gconf = g_conf_new();
+  gconf = g_conf_engine_new();
   
-  priv = (GConfPrivate*)gconf;
+  priv = (GConfEnginePrivate*)gconf;
 
   priv->context = ctx;
   priv->refcount = 1;
@@ -237,9 +179,9 @@ g_conf_new_from_address(const gchar* address)
 }
 
 void
-g_conf_ref             (GConf* conf)
+g_conf_engine_ref             (GConfEngine* conf)
 {
-  GConfPrivate* priv = (GConfPrivate*)conf;
+  GConfEnginePrivate* priv = (GConfEnginePrivate*)conf;
 
   g_return_if_fail(priv != NULL);
   g_return_if_fail(priv->refcount > 0);
@@ -248,10 +190,10 @@ g_conf_ref             (GConf* conf)
 }
 
 void         
-g_conf_unref        (GConf* conf)
+g_conf_engine_unref        (GConfEngine* conf)
 {
-  GConfPrivate* priv = (GConfPrivate*)conf;
-
+  GConfEnginePrivate* priv = (GConfEnginePrivate*)conf;
+  
   g_return_if_fail(priv != NULL);
   g_return_if_fail(priv->refcount > 0);
 
@@ -266,9 +208,12 @@ g_conf_unref        (GConf* conf)
       ConfigServer cs;
 
 
-      cs = g_conf_get_config_server(FALSE); /* don't restart it if down, since
-                                               the new one won't have the connections
-                                               to remove */
+      cs = g_conf_get_config_server(FALSE, NULL); /* don't restart it
+                                                     if down, since
+                                                     the new one won't
+                                                     have the
+                                                     connections to
+                                                     remove */
 
       if (cs == CORBA_OBJECT_NIL)
         g_warning("Config server is down while destroying GConf %p", conf);
@@ -284,18 +229,20 @@ g_conf_unref        (GConf* conf)
 
           if (cs != CORBA_OBJECT_NIL)
             {
+              GConfError* err = NULL;
+              
               ConfigServer_remove_listener(cs,
                                            priv->context,
                                            gcnxn->server_id,
                                            &ev);
 
-              if (g_conf_handle_corba_exception(&ev))
+              if (g_conf_handle_corba_exception(&ev, &err))
                 {
                   /* Don't set error because realistically this doesn't matter to 
                      clients */
                   g_warning("Failure removing listener %u from the config server: %s",
                             (guint)gcnxn->server_id,
-                            g_conf_error());
+                            err->str);
                 }
             }
 
@@ -309,19 +256,20 @@ g_conf_unref        (GConf* conf)
 }
 
 guint
-g_conf_notify_add(GConf* conf,
+g_conf_notify_add(GConfEngine* conf,
                   const gchar* namespace_section, /* dir or key to listen to */
                   GConfNotifyFunc func,
-                  gpointer user_data)
+                  gpointer user_data,
+                  GConfError** err)
 {
-  GConfPrivate* priv = (GConfPrivate*)conf;
+  GConfEnginePrivate* priv = (GConfEnginePrivate*)conf;
   ConfigServer cs;
   ConfigListener cl;
   gulong id;
   CORBA_Environment ev;
   GConfCnxn* cnxn;
 
-  cs = g_conf_get_config_server(TRUE);
+  cs = g_conf_get_config_server(TRUE, err);
 
   if (cs == CORBA_OBJECT_NIL)
     return 0;
@@ -337,7 +285,7 @@ g_conf_notify_add(GConf* conf,
                                  (gchar*)namespace_section, 
                                  cl, &ev);
 
-  if (g_conf_handle_corba_exception(&ev))
+  if (g_conf_handle_corba_exception(&ev, err))
     {
       /* FIXME we could do better here... maybe respawn the server if needed... */
       return 0;
@@ -354,15 +302,15 @@ g_conf_notify_add(GConf* conf,
 }
 
 void         
-g_conf_notify_remove(GConf* conf,
+g_conf_notify_remove(GConfEngine* conf,
                      guint client_id)
 {
-  GConfPrivate* priv = (GConfPrivate*)conf;
+  GConfEnginePrivate* priv = (GConfEnginePrivate*)conf;
   GConfCnxn* gcnxn;
   CORBA_Environment ev;
   ConfigServer cs;
 
-  cs = g_conf_get_config_server(TRUE);
+  cs = g_conf_get_config_server(TRUE, NULL);
 
   if (cs == CORBA_OBJECT_NIL)
     return;
@@ -377,7 +325,7 @@ g_conf_notify_remove(GConf* conf,
                                gcnxn->server_id,
                                &ev);
 
-  if (g_conf_handle_corba_exception(&ev))
+  if (g_conf_handle_corba_exception(&ev, NULL))
     {
       /* FIXME we could do better here... maybe respawn the server if needed... */
     }
@@ -391,26 +339,22 @@ g_conf_notify_remove(GConf* conf,
 }
 
 GConfValue*  
-g_conf_get(GConf* conf, const gchar* key)
+g_conf_get(GConfEngine* conf, const gchar* key, GConfError** err)
 {
-  GConfPrivate* priv = (GConfPrivate*)conf;
+  GConfEnginePrivate* priv = (GConfEnginePrivate*)conf;
   GConfValue* val;
   ConfigValue* cv;
   CORBA_Environment ev;
   ConfigServer cs;
 
-  if (!g_conf_valid_key(key))
-    {
-      g_conf_set_error(G_CONF_BAD_KEY, _("`%s'"), key);
-      
-      return NULL;
-    }
+  if (!g_conf_key_check(key, err))
+    return NULL;
 
-  cs = g_conf_get_config_server(TRUE);
+  cs = g_conf_get_config_server(TRUE, err);
 
   if (cs == CORBA_OBJECT_NIL)
     {
-      g_assert(g_conf_errno() == G_CONF_NO_SERVER);
+      g_return_val_if_fail(((err == NULL) || ((*err)->num == G_CONF_NO_SERVER)), NULL);
       return NULL;
     }
 
@@ -418,7 +362,7 @@ g_conf_get(GConf* conf, const gchar* key)
   
   cv = ConfigServer_lookup(cs, priv->context, (gchar*)key, &ev);
 
-  if (g_conf_handle_corba_exception(&ev))
+  if (g_conf_handle_corba_exception(&ev, err))
     {
       /* FIXME we could do better here... maybe respawn the server if needed... */
 
@@ -435,26 +379,23 @@ g_conf_get(GConf* conf, const gchar* key)
 }
 
 void
-g_conf_set(GConf* conf, const gchar* key, GConfValue* value)
+g_conf_set(GConfEngine* conf, const gchar* key, GConfValue* value, GConfError** err)
 {
-  GConfPrivate* priv = (GConfPrivate*)conf;
+  GConfEnginePrivate* priv = (GConfEnginePrivate*)conf;
   ConfigValue* cv;
   CORBA_Environment ev;
   ConfigServer cs;
 
   g_return_if_fail(value->type != G_CONF_VALUE_INVALID);
 
-  if (!g_conf_valid_key(key))
-    {
-      g_conf_set_error(G_CONF_BAD_KEY, _("`%s'"), key);
-      return;
-    }
+  if (!g_conf_key_check(key, err))
+    return;
 
-  cs = g_conf_get_config_server(TRUE);
+  cs = g_conf_get_config_server(TRUE, err);
 
   if (cs == CORBA_OBJECT_NIL)
     {
-      g_assert(g_conf_errno() == G_CONF_NO_SERVER);
+      g_return_if_fail(((err == NULL) || ((*err)->num == G_CONF_NO_SERVER)));
       return;
     }
 
@@ -468,30 +409,27 @@ g_conf_set(GConf* conf, const gchar* key, GConfValue* value)
 
   CORBA_free(cv);
 
-  if (g_conf_handle_corba_exception(&ev))
+  if (g_conf_handle_corba_exception(&ev, err))
     {
       /* FIXME we could do better here... maybe respawn the server if needed... */
     }
 }
 
 void         
-g_conf_unset(GConf* conf, const gchar* key)
+g_conf_unset(GConfEngine* conf, const gchar* key, GConfError** err)
 {
-  GConfPrivate* priv = (GConfPrivate*)conf;
+  GConfEnginePrivate* priv = (GConfEnginePrivate*)conf;
   CORBA_Environment ev;
   ConfigServer cs;
 
-  if (!g_conf_valid_key(key))
-    {
-      g_conf_set_error(G_CONF_BAD_KEY, _("`%s'"), key);
-      return;
-    }
+  if (!g_conf_key_check(key, err))
+    return;
 
-  cs = g_conf_get_config_server(TRUE);
+  cs = g_conf_get_config_server(TRUE, err);
 
   if (cs == CORBA_OBJECT_NIL)
     {
-      g_assert(g_conf_errno() == G_CONF_NO_SERVER);
+      g_return_if_fail(((err == NULL) || ((*err)->num == G_CONF_NO_SERVER)));
       return;
     }
 
@@ -501,16 +439,16 @@ g_conf_unset(GConf* conf, const gchar* key)
                      (gchar*)key,
                      &ev);
 
-  if (g_conf_handle_corba_exception(&ev))
+  if (g_conf_handle_corba_exception(&ev, err))
     {
       /* FIXME we could do better here... maybe respawn the server if needed... */
     }
 }
 
 GSList*      
-g_conf_all_entries(GConf* conf, const gchar* dir)
+g_conf_all_entries(GConfEngine* conf, const gchar* dir, GConfError** err)
 {
-  GConfPrivate* priv = (GConfPrivate*)conf;
+  GConfEnginePrivate* priv = (GConfEnginePrivate*)conf;
   GSList* pairs = NULL;
   ConfigServer_ValueList* values;
   ConfigServer_KeyList* keys;
@@ -518,17 +456,14 @@ g_conf_all_entries(GConf* conf, const gchar* dir)
   ConfigServer cs;
   guint i;
 
-  if (!g_conf_valid_key(dir))
-    {
-      g_conf_set_error(G_CONF_BAD_KEY, _("`%s'"), dir);
-      return NULL;
-    }
+  if (!g_conf_key_check(dir, err))
+    return NULL;
 
-  cs = g_conf_get_config_server(TRUE);
+  cs = g_conf_get_config_server(TRUE, err);
 
   if (cs == CORBA_OBJECT_NIL)
     {
-      g_assert(g_conf_errno() == G_CONF_NO_SERVER);
+      g_return_val_if_fail(((err == NULL) || ((*err)->num == G_CONF_NO_SERVER)), NULL);
       return NULL;
     }
 
@@ -539,7 +474,7 @@ g_conf_all_entries(GConf* conf, const gchar* dir)
                            &keys, &values,
                            &ev);
 
-  if (g_conf_handle_corba_exception(&ev))
+  if (g_conf_handle_corba_exception(&ev, err))
     {
       /* FIXME we could do better here... maybe respawn the server if needed... */
 
@@ -574,26 +509,23 @@ g_conf_all_entries(GConf* conf, const gchar* dir)
 }
 
 GSList*      
-g_conf_all_dirs(GConf* conf, const gchar* dir)
+g_conf_all_dirs(GConfEngine* conf, const gchar* dir, GConfError** err)
 {
-  GConfPrivate* priv = (GConfPrivate*)conf;
+  GConfEnginePrivate* priv = (GConfEnginePrivate*)conf;
   GSList* subdirs = NULL;
   ConfigServer_KeyList* keys;
   CORBA_Environment ev;
   ConfigServer cs;
   guint i;
 
-  if (!g_conf_valid_key(dir))
-    {
-      g_conf_set_error(G_CONF_BAD_KEY, _("`%s'"), dir);
-      return NULL;
-    }
+  if (!g_conf_key_check(dir, err))
+    return NULL;
 
-  cs = g_conf_get_config_server(TRUE);
+  cs = g_conf_get_config_server(TRUE, err);
 
   if (cs == CORBA_OBJECT_NIL)
     {
-      g_assert(g_conf_errno() == G_CONF_NO_SERVER);
+      g_return_val_if_fail(((err == NULL) || ((*err)->num == G_CONF_NO_SERVER)), NULL);
       return NULL;
     }
 
@@ -605,7 +537,7 @@ g_conf_all_dirs(GConf* conf, const gchar* dir)
                         &ev);
 
 
-  if (g_conf_handle_corba_exception(&ev))
+  if (g_conf_handle_corba_exception(&ev, err))
     {
       /* FIXME we could do better here... maybe respawn the server if needed... */
 
@@ -630,17 +562,17 @@ g_conf_all_dirs(GConf* conf, const gchar* dir)
 }
 
 void 
-g_conf_sync(GConf* conf)
+g_conf_sync(GConfEngine* conf, GConfError** err)
 {
-  GConfPrivate* priv = (GConfPrivate*)conf;
+  GConfEnginePrivate* priv = (GConfEnginePrivate*)conf;
   CORBA_Environment ev;
   ConfigServer cs;
 
-  cs = g_conf_get_config_server(TRUE);
+  cs = g_conf_get_config_server(TRUE, err);
 
   if (cs == CORBA_OBJECT_NIL)
     {
-      g_assert(g_conf_errno() == G_CONF_NO_SERVER);
+      g_return_if_fail(((err == NULL) || ((*err)->num == G_CONF_NO_SERVER)));
       return;
     }
 
@@ -648,33 +580,30 @@ g_conf_sync(GConf* conf)
 
   ConfigServer_sync(cs, priv->context, &ev);
 
-  if (g_conf_handle_corba_exception(&ev))  
+  if (g_conf_handle_corba_exception(&ev, err))  
     {
       /* FIXME we could do better here... maybe respawn the server if needed... */
     }
 }
 
 gboolean
-g_conf_dir_exists(GConf *conf, const gchar *dir)
+g_conf_dir_exists(GConfEngine *conf, const gchar *dir, GConfError** err)
 {
-  GConfPrivate* priv = (GConfPrivate*)conf;
+  GConfEnginePrivate* priv = (GConfEnginePrivate*)conf;
   CORBA_Environment ev;
   ConfigServer cs;
   CORBA_boolean server_ret;
 
   g_return_val_if_fail(dir != NULL, FALSE);
   
-  if (!g_conf_valid_key(dir))
-    {
-      g_conf_set_error(G_CONF_BAD_KEY, _("`%s'"), dir);
-      return FALSE;
-    }
+  if (!g_conf_key_check(dir, err))
+    return FALSE;
   
-  cs = g_conf_get_config_server(TRUE);
+  cs = g_conf_get_config_server(TRUE, err);
   
   if (cs == CORBA_OBJECT_NIL)
     {
-      g_assert(g_conf_errno() == G_CONF_NO_SERVER);
+      g_return_val_if_fail(((err == NULL) || ((*err)->num == G_CONF_NO_SERVER)), FALSE);
       return FALSE;
     }
   
@@ -683,7 +612,7 @@ g_conf_dir_exists(GConf *conf, const gchar *dir)
   server_ret = ConfigServer_dir_exists(cs, priv->context,
                                        (gchar*)dir, &ev);
   
-  if (g_conf_handle_corba_exception(&ev))  
+  if (g_conf_handle_corba_exception(&ev, err))  
     {
       /* FIXME we could do better here... maybe respawn the server if needed... */
     }
@@ -696,7 +625,7 @@ g_conf_dir_exists(GConf *conf, const gchar *dir)
  */
 
 static GConfCnxn* 
-g_conf_cnxn_new(GConf* conf, CORBA_unsigned_long server_id, GConfNotifyFunc func, gpointer user_data)
+g_conf_cnxn_new(GConfEngine* conf, CORBA_unsigned_long server_id, GConfNotifyFunc func, gpointer user_data)
 {
   GConfCnxn* cnxn;
   static guint next_id = 1;
@@ -735,14 +664,17 @@ static ConfigServer   server = CORBA_OBJECT_NIL;
 
 /* errors in here should be G_CONF_NO_SERVER */
 static ConfigServer
-try_to_contact_server(void)
+try_to_contact_server(GConfError** err)
 {
   /* This writing-IOR-to-file crap is a temporary hack. */
   gchar* ior;
 
   g_return_val_if_fail(server == CORBA_OBJECT_NIL, server);
   
-  ior = g_conf_read_server_ior();
+  ior = g_conf_read_server_ior(err);
+
+  if (ior == NULL)
+    return CORBA_OBJECT_NIL;
   
   if (ior != NULL)
     {
@@ -761,29 +693,31 @@ try_to_contact_server(void)
           if (ev._major != CORBA_NO_EXCEPTION)
             {
               server = CORBA_OBJECT_NIL;
-              g_conf_set_error(G_CONF_NO_SERVER, _("Pinging the server failed, CORBA error: %s"),
-                               CORBA_exception_id(&ev));
+              if (err)
+                *err = g_conf_error_new(G_CONF_NO_SERVER, _("Pinging the server failed, CORBA error: %s"),
+                                        CORBA_exception_id(&ev));
               CORBA_exception_free(&ev);
             }
         }
-      else 
-        g_conf_set_error(G_CONF_NO_SERVER, _("Failed to convert server IOR to an object reference"));
+      else
+        {
+          if (err)
+            *err = g_conf_error_new(G_CONF_NO_SERVER, _("Failed to convert server IOR to an object reference"));
+        }
     }
-  else
-    g_conf_set_error(G_CONF_NO_SERVER, _("Failed to read the server's IOR"));
-
+      
   return server;
 }
 
 /* All errors set in here should be G_CONF_NO_SERVER; should
    only set errors if start_if_not_found is TRUE */
 static ConfigServer
-g_conf_get_config_server(gboolean start_if_not_found)
+g_conf_get_config_server(gboolean start_if_not_found, GConfError** err)
 {
   if (server != CORBA_OBJECT_NIL)
     return server;
   
-  server = try_to_contact_server();
+  server = try_to_contact_server(err);
 
   if (!start_if_not_found)
     return server;
@@ -796,8 +730,9 @@ g_conf_get_config_server(gboolean start_if_not_found)
 
       if (pipe(fds) < 0)
         {
-          g_conf_set_error(G_CONF_NO_SERVER, _("Failed to create pipe to server: %s"),
-                           strerror(errno));
+          if (err)
+            *err = g_conf_error_new(G_CONF_NO_SERVER, _("Failed to create pipe to server: %s"),
+                                    strerror(errno));
           return CORBA_OBJECT_NIL;
         }
 
@@ -805,8 +740,9 @@ g_conf_get_config_server(gboolean start_if_not_found)
           
       if (pid < 0)
         {
-          g_conf_set_error(G_CONF_NO_SERVER, _("gconfd fork failed: %s"), 
-                           strerror(errno));
+          if (err)
+            *err = g_conf_error_new(G_CONF_NO_SERVER, _("gconfd fork failed: %s"), 
+                                    strerror(errno));
           return CORBA_OBJECT_NIL;
         }
           
@@ -833,9 +769,10 @@ g_conf_get_config_server(gboolean start_if_not_found)
       /* Parent - waitpid(), gconfd instantly forks anyway */
       if (waitpid(pid, &status, 0) != pid)
         {
-          g_conf_set_error(G_CONF_NO_SERVER, 
-                           _("waitpid() failed waiting for child in %s: %s"),
-                           __FUNCTION__, strerror(errno));
+          if (err)
+            *err = g_conf_error_new(G_CONF_NO_SERVER, 
+                                    _("waitpid() failed waiting for child in %s: %s"),
+                                    __FUNCTION__, strerror(errno));
           close(fds[1]);
           return CORBA_OBJECT_NIL;
         }
@@ -844,15 +781,17 @@ g_conf_get_config_server(gboolean start_if_not_found)
         {
           if (WEXITSTATUS(status) != 0)
             {
-              g_conf_set_error(G_CONF_NO_SERVER, 
-                               _("spawned server returned error code, giving up on contacting it."));
+              if (err)
+                *err = g_conf_error_new(G_CONF_NO_SERVER, 
+                                        _("spawned server returned error code, giving up on contacting it."));
               close(fds[1]);
               return CORBA_OBJECT_NIL;
             }
         }
       else
         {
-          g_conf_set_error(G_CONF_NO_SERVER, _("spawned gconfd child didn't exit normally, can't contact server."));
+          if (err)
+            *err = g_conf_error_new(G_CONF_NO_SERVER, _("spawned gconfd child didn't exit normally, can't contact server."));
           close(fds[1]);
           return CORBA_OBJECT_NIL;
         }
@@ -879,7 +818,7 @@ g_conf_get_config_server(gboolean start_if_not_found)
         close(fds[0]);
       }
 
-      server = try_to_contact_server();
+      server = try_to_contact_server(err);
     }
 
   return server;
@@ -941,19 +880,9 @@ g_conf_get_config_listener(void)
 }
 
 static gboolean have_initted = FALSE;
-static gchar* global_appname = NULL;
-
-const gchar*
-g_conf_global_appname(void)
-{
-  if (global_appname)
-    return global_appname;
-  else
-    return "";
-}
 
 gboolean     
-g_conf_init           (const gchar* appname)
+g_conf_init           (GConfError** err)
 {
   static CORBA_ORB orb = CORBA_OBJECT_NIL;
 
@@ -985,8 +914,9 @@ g_conf_init           (const gchar* appname)
       
       if (ev._major != CORBA_NO_EXCEPTION)
         {
-          g_conf_set_error(G_CONF_FAILED, _("Failed to init the listener servant: %s"),
-                           CORBA_exception_id(&ev));
+          if (err)
+            *err = g_conf_error_new(G_CONF_FAILED, _("Failed to init the listener servant: %s"),
+                                    CORBA_exception_id(&ev));
           return FALSE;
         }
 
@@ -994,8 +924,9 @@ g_conf_init           (const gchar* appname)
 
       if (ev._major != CORBA_NO_EXCEPTION)
         {
-          g_conf_set_error(G_CONF_FAILED, _("Failed to resolve the root POA: %s"),
-                           CORBA_exception_id(&ev));
+          if (err)
+            *err = g_conf_error_new(G_CONF_FAILED, _("Failed to resolve the root POA: %s"),
+                                    CORBA_exception_id(&ev));
           return FALSE;
         }
 
@@ -1004,8 +935,9 @@ g_conf_init           (const gchar* appname)
 
       if (ev._major != CORBA_NO_EXCEPTION)
         {
-          g_conf_set_error(G_CONF_FAILED, _("Failed to activate the POA Manager: %s"),
-                           CORBA_exception_id(&ev));
+          if (err)
+            *err = g_conf_error_new(G_CONF_FAILED, _("Failed to activate the POA Manager: %s"),
+                                    CORBA_exception_id(&ev));
           return FALSE;
         }
 
@@ -1014,8 +946,9 @@ g_conf_init           (const gchar* appname)
 
       if (ev._major != CORBA_NO_EXCEPTION)
         {
-          g_conf_set_error(G_CONF_FAILED, _("Failed to activate the listener servant: %s"),
-                           CORBA_exception_id(&ev));
+          if (err)
+            *err = g_conf_error_new(G_CONF_FAILED, _("Failed to activate the listener servant: %s"),
+                                    CORBA_exception_id(&ev));
           return FALSE;
         }
 
@@ -1026,23 +959,22 @@ g_conf_init           (const gchar* appname)
 
       if (listener == CORBA_OBJECT_NIL || ev._major != CORBA_NO_EXCEPTION)
         {
-          g_conf_set_error(G_CONF_FAILED, _("Failed get object reference for the listener servant: %s"),
-                           CORBA_exception_id(&ev));
+          if (err)
+            *err = g_conf_error_new(G_CONF_FAILED, _("Failed get object reference for the listener servant: %s"),
+                                    CORBA_exception_id(&ev));
           return FALSE;
         }
     }
 
   ctable = ctable_new();
 
-  global_appname = g_strdup(appname);
-  
   return TRUE;
 }
 
 gboolean
 g_conf_is_initialized (void)
 {
-        return (have_initted);
+  return (have_initted);
 }
 
 /* 
@@ -1058,14 +990,18 @@ g_conf_is_initialized (void)
 static const gchar invalid_chars[] = "\"$&<>,+=#!()'|{}[]?~`;%\\";
 
 gboolean     
-g_conf_valid_key      (const gchar* key)
+g_conf_valid_key      (const gchar* key, gchar** why_invalid)
 {
   const gchar* s = key;
   gboolean just_saw_slash = FALSE;
 
   /* Key must start with the root */
   if (*key != '/')
-    return FALSE;
+    {
+      if (why_invalid != NULL)
+        *why_invalid = g_strdup(_("Must begin with a slash (/)"));
+      return FALSE;
+    }
   
   /* Root key is a valid dir */
   if (*key == '/' && key[1] == '\0')
@@ -1081,7 +1017,16 @@ g_conf_valid_key      (const gchar* key)
            * because it would be a pain for filesystem-based backends.
            */
           if (*s == '/' || *s == '.')
-            return FALSE;
+            {
+              if (why_invalid != NULL)
+                {
+                  if (*s == '/')
+                    *why_invalid = g_strdup(_("Can't have two slashes (/) in a row"));
+                  else
+                    *why_invalid = g_strdup(_("Can't have a period (.) right after a slash (/)"));
+                }
+              return FALSE;
+            }
         }
 
       if (*s == '/')
@@ -1097,7 +1042,11 @@ g_conf_valid_key      (const gchar* key)
           while (*inv)
             {
               if (*inv == *s)
-                return FALSE;
+                {
+                  if (why_invalid != NULL)
+                    *why_invalid = g_strdup_printf(_("`%c' is an invalid character in key/directory names"), *s);
+                  return FALSE;
+                }
               ++inv;
             }
         }
@@ -1107,7 +1056,11 @@ g_conf_valid_key      (const gchar* key)
 
   /* Can't end with slash */
   if (just_saw_slash)
-    return FALSE;
+    {
+      if (why_invalid != NULL)
+        *why_invalid = g_strdup(_("Key/directory may not end with a slash (/)"));
+      return FALSE;
+    }
   else
     return TRUE;
 }
@@ -1171,7 +1124,7 @@ ctable_remove_by_client_id(CnxnTable* ct, guint client_id)
 
 struct RemoveData {
   GSList* removed;
-  GConf* conf;
+  GConfEngine* conf;
   gboolean save_removed;
 };
 
@@ -1194,7 +1147,7 @@ remove_by_conf(gpointer key, gpointer value, gpointer user_data)
 
 /* We return a list of the removed GConfCnxn */
 static GSList*      
-ctable_remove_by_conf(CnxnTable* ct, GConf* conf)
+ctable_remove_by_conf(CnxnTable* ct, GConfEngine* conf)
 {
   guint client_ids_removed;
   guint server_ids_removed;
@@ -1234,16 +1187,16 @@ ctable_lookup_by_server_id(CnxnTable* ct, CORBA_unsigned_long server_id)
  */
 
 void          
-g_conf_shutdown_daemon(void)
+g_conf_shutdown_daemon(GConfError** err)
 {
   CORBA_Environment ev;
   ConfigServer cs;
 
-  cs = g_conf_get_config_server(FALSE); /* Don't want to spawn it if it's already down */
+  cs = g_conf_get_config_server(FALSE, err); /* Don't want to spawn it if it's already down */
 
   if (cs == CORBA_OBJECT_NIL)
     {
-      g_assert(g_conf_errno() == G_CONF_NO_SERVER);
+      g_return_if_fail(((err == NULL) || ((*err)->num == G_CONF_NO_SERVER)));
       return;
     }
 
@@ -1253,8 +1206,9 @@ g_conf_shutdown_daemon(void)
 
   if (ev._major != CORBA_NO_EXCEPTION)
     {
-      g_conf_set_error(G_CONF_FAILED, _("Failure shutting down config server: %s"),
-                       CORBA_exception_id(&ev));
+      if (err)
+        *err = g_conf_error_new(G_CONF_FAILED, _("Failure shutting down config server: %s"),
+                                CORBA_exception_id(&ev));
       /* FIXME we could do better here... maybe respawn the server if needed... */
       CORBA_exception_free(&ev);
     }
@@ -1264,26 +1218,27 @@ gboolean
 g_conf_ping_daemon(void)
 {
   ConfigServer cs;
-
-  cs = g_conf_get_config_server(FALSE);
+  
+  cs = g_conf_get_config_server(FALSE, NULL); /* ignore error, since whole point is to see if server is reachable */
 
   if (cs == CORBA_OBJECT_NIL)
-    return FALSE; /* FIXME this means an error was set, but I guess it
-                     doesn't matter; errno works the same way (random
-                     error setting). */
+    return FALSE;
   else
     return TRUE;
 }
 
 gboolean
-g_conf_spawn_daemon(void)
+g_conf_spawn_daemon(GConfError** err)
 {
   ConfigServer cs;
 
-  cs = g_conf_get_config_server(TRUE);
+  cs = g_conf_get_config_server(TRUE, err);
 
   if (cs == CORBA_OBJECT_NIL)
-    return FALSE; /* Failed to spawn, error should be set */
+    {
+      g_return_val_if_fail(err == NULL || *err != NULL, FALSE);
+      return FALSE; /* Failed to spawn, error should be set */
+    }
   else
     return TRUE;
 }
@@ -1293,12 +1248,12 @@ g_conf_spawn_daemon(void)
  */
 
 gdouble      
-g_conf_get_float (GConf* conf, const gchar* key,
-                  gdouble deflt)
+g_conf_get_float (GConfEngine* conf, const gchar* key,
+                  gdouble deflt, GConfError** err)
 {
   GConfValue* val;
 
-  val = g_conf_get(conf, key);
+  val = g_conf_get(conf, key, err);
 
   if (val == NULL)
     return deflt;
@@ -1308,8 +1263,9 @@ g_conf_get_float (GConf* conf, const gchar* key,
       
       if (val->type != G_CONF_VALUE_FLOAT)
         {
-          g_conf_set_error(G_CONF_TYPE_MISMATCH, _("Expected float, got %s"),
-                           g_conf_value_type_to_string(val->type));
+          if (err)
+            *err = g_conf_error_new(G_CONF_TYPE_MISMATCH, _("Expected float, got %s"),
+                                    g_conf_value_type_to_string(val->type));
           g_conf_value_destroy(val);
           return deflt;
         }
@@ -1323,12 +1279,12 @@ g_conf_get_float (GConf* conf, const gchar* key,
 }
 
 gint         
-g_conf_get_int   (GConf* conf, const gchar* key,
-                  gint deflt)
+g_conf_get_int   (GConfEngine* conf, const gchar* key,
+                  gint deflt, GConfError** err)
 {
   GConfValue* val;
 
-  val = g_conf_get(conf, key);
+  val = g_conf_get(conf, key, err);
 
   if (val == NULL)
     return deflt;
@@ -1338,8 +1294,9 @@ g_conf_get_int   (GConf* conf, const gchar* key,
 
       if (val->type != G_CONF_VALUE_INT)
         {
-          g_conf_set_error(G_CONF_TYPE_MISMATCH, _("Expected int, got %s"),
-                           g_conf_value_type_to_string(val->type));
+          if (err)
+            *err = g_conf_error_new(G_CONF_TYPE_MISMATCH, _("Expected int, got %s"),
+                                    g_conf_value_type_to_string(val->type));
           g_conf_value_destroy(val);
           return deflt;
         }
@@ -1353,12 +1310,12 @@ g_conf_get_int   (GConf* conf, const gchar* key,
 }
 
 gchar*       
-g_conf_get_string(GConf* conf, const gchar* key,
-                  const gchar* deflt)
+g_conf_get_string(GConfEngine* conf, const gchar* key,
+                  const gchar* deflt, GConfError** err)
 {
   GConfValue* val;
 
-  val = g_conf_get(conf, key);
+  val = g_conf_get(conf, key, err);
 
   if (val == NULL)
     return deflt ? g_strdup(deflt) : NULL;
@@ -1368,8 +1325,9 @@ g_conf_get_string(GConf* conf, const gchar* key,
 
       if (val->type != G_CONF_VALUE_STRING)
         {
-          g_conf_set_error(G_CONF_TYPE_MISMATCH, _("Expected string, got %s"),
-                           g_conf_value_type_to_string(val->type));
+          if (err)
+            *err = g_conf_error_new(G_CONF_TYPE_MISMATCH, _("Expected string, got %s"),
+                                    g_conf_value_type_to_string(val->type));
           g_conf_value_destroy(val);
           return deflt ? g_strdup(deflt) : NULL;
         }
@@ -1386,12 +1344,12 @@ g_conf_get_string(GConf* conf, const gchar* key,
 }
 
 gboolean     
-g_conf_get_bool  (GConf* conf, const gchar* key,
-                  gboolean deflt)
+g_conf_get_bool  (GConfEngine* conf, const gchar* key,
+                  gboolean deflt, GConfError** err)
 {
   GConfValue* val;
 
-  val = g_conf_get(conf, key);
+  val = g_conf_get(conf, key, err);
 
   if (val == NULL)
     return deflt;
@@ -1401,8 +1359,9 @@ g_conf_get_bool  (GConf* conf, const gchar* key,
 
       if (val->type != G_CONF_VALUE_BOOL)
         {
-          g_conf_set_error(G_CONF_TYPE_MISMATCH, _("Expected bool, got %s"),
-                           g_conf_value_type_to_string(val->type));
+          if (err)
+            *err = g_conf_error_new(G_CONF_TYPE_MISMATCH, _("Expected bool, got %s"),
+                                    g_conf_value_type_to_string(val->type));
           g_conf_value_destroy(val);
           return deflt;
         }
@@ -1416,11 +1375,11 @@ g_conf_get_bool  (GConf* conf, const gchar* key,
 }
 
 GConfSchema* 
-g_conf_get_schema  (GConf* conf, const gchar* key)
+g_conf_get_schema  (GConfEngine* conf, const gchar* key, GConfError** err)
 {
   GConfValue* val;
 
-  val = g_conf_get(conf, key);
+  val = g_conf_get(conf, key, err);
 
   if (val == NULL)
     return NULL;
@@ -1430,8 +1389,9 @@ g_conf_get_schema  (GConf* conf, const gchar* key)
 
       if (val->type != G_CONF_VALUE_SCHEMA)
         {
-          g_conf_set_error(G_CONF_TYPE_MISMATCH, _("Expected schema, got %s"),
-                           g_conf_value_type_to_string(val->type));
+          if (err)
+            *err = g_conf_error_new(G_CONF_TYPE_MISMATCH, _("Expected schema, got %s"),
+                                    g_conf_value_type_to_string(val->type));
           g_conf_value_destroy(val);
           return NULL;
         }
@@ -1452,24 +1412,28 @@ g_conf_get_schema  (GConf* conf, const gchar* key)
  */
 
 static gboolean
-error_checked_set(GConf* conf, const gchar* key,
-                  GConfValue* gval)
+error_checked_set(GConfEngine* conf, const gchar* key,
+                  GConfValue* gval, GConfError** err)
 {
-  g_conf_clear_error();
+  GConfError* my_err = NULL;
   
-  g_conf_set(conf, key, gval);
+  g_conf_set(conf, key, gval, &my_err);
 
   g_conf_value_destroy(gval);
   
-  if (g_conf_errno() != G_CONF_SUCCESS)
-    return FALSE;
+  if (my_err != NULL)
+    {
+      if (err)
+        *err = my_err;
+      return FALSE;
+    }
   else
     return TRUE;
 }
 
 gboolean
-g_conf_set_float   (GConf* conf, const gchar* key,
-                    gdouble val)
+g_conf_set_float   (GConfEngine* conf, const gchar* key,
+                    gdouble val, GConfError** err)
 {
   GConfValue* gval;
 
@@ -1477,12 +1441,12 @@ g_conf_set_float   (GConf* conf, const gchar* key,
 
   g_conf_value_set_float(gval, val);
 
-  return error_checked_set(conf, key, gval);
+  return error_checked_set(conf, key, gval, err);
 }
 
 gboolean
-g_conf_set_int     (GConf* conf, const gchar* key,
-                    gint val)
+g_conf_set_int     (GConfEngine* conf, const gchar* key,
+                    gint val, GConfError** err)
 {
   GConfValue* gval;
 
@@ -1490,12 +1454,12 @@ g_conf_set_int     (GConf* conf, const gchar* key,
 
   g_conf_value_set_int(gval, val);
 
-  return error_checked_set(conf, key, gval);
+  return error_checked_set(conf, key, gval, err);
 }
 
 gboolean
-g_conf_set_string  (GConf* conf, const gchar* key,
-                    const gchar* val)
+g_conf_set_string  (GConfEngine* conf, const gchar* key,
+                    const gchar* val, GConfError** err)
 {
   GConfValue* gval;
 
@@ -1503,12 +1467,12 @@ g_conf_set_string  (GConf* conf, const gchar* key,
 
   g_conf_value_set_string(gval, val);
 
-  return error_checked_set(conf, key, gval);
+  return error_checked_set(conf, key, gval, err);
 }
 
 gboolean
-g_conf_set_bool    (GConf* conf, const gchar* key,
-                    gboolean val)
+g_conf_set_bool    (GConfEngine* conf, const gchar* key,
+                    gboolean val, GConfError** err)
 {
   GConfValue* gval;
 
@@ -1516,12 +1480,12 @@ g_conf_set_bool    (GConf* conf, const gchar* key,
 
   g_conf_value_set_bool(gval, val);
 
-  return error_checked_set(conf, key, gval);
+  return error_checked_set(conf, key, gval, err);
 }
 
 gboolean
-g_conf_set_schema  (GConf* conf, const gchar* key,
-                    GConfSchema* val)
+g_conf_set_schema  (GConfEngine* conf, const gchar* key,
+                    GConfSchema* val, GConfError** err)
 {
   GConfValue* gval;
 
@@ -1529,7 +1493,7 @@ g_conf_set_schema  (GConf* conf, const gchar* key,
 
   g_conf_value_set_schema(gval, val);
 
-  return error_checked_set(conf, key, gval);
+  return error_checked_set(conf, key, gval, err);
 }
 
 /* CORBA Util */
@@ -1579,7 +1543,7 @@ corba_errno_to_g_conf_errno(ConfigErrorType corba_err)
 }
 
 static gboolean
-g_conf_handle_corba_exception(CORBA_Environment* ev)
+g_conf_handle_corba_exception(CORBA_Environment* ev, GConfError** err)
 {
   switch (ev->_major)
     {
@@ -1588,8 +1552,9 @@ g_conf_handle_corba_exception(CORBA_Environment* ev)
       return FALSE;
       break;
     case CORBA_SYSTEM_EXCEPTION:
-      g_conf_set_error(G_CONF_NO_SERVER, _("CORBA error: %s"),
-                       CORBA_exception_id(ev));
+      if (err)
+        *err = g_conf_error_new(G_CONF_NO_SERVER, _("CORBA error: %s"),
+                                CORBA_exception_id(ev));
       CORBA_exception_free(ev);
       return TRUE;
       break;
@@ -1599,8 +1564,9 @@ g_conf_handle_corba_exception(CORBA_Environment* ev)
 
         ce = CORBA_exception_value(ev);
 
-        g_conf_set_error(corba_errno_to_g_conf_errno(ce->err_no),
-                         ce->message);
+        if (err)
+          *err = g_conf_error_new(corba_errno_to_g_conf_errno(ce->err_no),
+                                  ce->message);
         CORBA_exception_free(ev);
         return TRUE;
       }
